@@ -3,23 +3,14 @@ import json
 import re
 from concurrent.futures import ThreadPoolExecutor, as_completed
 from datetime import timedelta
+from pathlib import Path
 from typing import List, Tuple
 
 import requests
-from flask import (
-    Flask,
-    flash,
-    jsonify,
-    make_response,
-    redirect,
-    render_template,
-    request,
-    session,
-    url_for,
-)
+from flask import Flask, jsonify, make_response, render_template, request, url_for
 
 from .conf import config
-from .decorators import process_token, require_login
+from .decorators import process_token
 from .persistence import (
     create_token,
     delete_token,
@@ -46,6 +37,7 @@ from .source_sites import (
     title_source_url,
     title_thumbnail,
 )
+from .storages import storage
 
 config.load()
 
@@ -71,149 +63,6 @@ def _chapter_name(chapter: dict):
     return result
 
 
-@app.route("/following", methods=["GET"])
-@require_login
-def follows_view():
-    titles = get_followed_titles(session["user"]["id"])
-    for title in titles:
-        thumbnail = title_thumbnail(title["site"], title["id"])
-        if title["site"] == "mangadex":
-            thumbnail = url_for("proxy_view", b64_url=_encode_proxy_url(thumbnail))
-        title["thumbnail"] = thumbnail
-    return render_template("old/follows.html", titles=titles)
-
-
-@app.route("/follow/<site>/<title_id>", methods=["POST"])
-@require_login
-def follow_view(site, title_id):
-    follow(session["user"]["id"], site, title_id)
-    return redirect(url_for("spa_title_view", site=site, title_id=title_id))
-
-
-@app.route("/unfollow/<site>/<title_id>", methods=["POST"])
-@require_login
-def unfollow_view(site, title_id):
-    unfollow(session["user"]["id"], site, title_id)
-    return redirect(url_for("spa_title_view", site=site, title_id=title_id))
-
-
-@app.route("/logout", methods=["POST"])
-def logout_view():
-    session.pop("user")
-    return redirect("/")
-
-
-@app.route("/auth", methods=["GET", "POST"])
-def auth_view():
-    if session.get("user"):
-        return redirect(url_for("home_view"))
-
-    if request.method == "POST":
-
-        if request.form["action"] == "register":
-            username = request.form["username"].strip()
-            password = request.form["password"]
-            confirm_password = request.form["confirm-password"]
-            message = None
-            if password != confirm_password:
-                message = "Password confirmation didn't match."
-                status_code = 400
-            elif not (username and password and confirm_password):
-                message = "Empty field(s) spotted. Protip: spaces don't count."
-                status_code = 400
-            elif (
-                len(username) < 2
-                or len(username) > 15
-                or len(password) < 5
-                or len(password) > 50
-            ):
-                message = "Invalid username/password length. Username length should be 2~15, password 5~50."
-                status_code = 400
-            else:  # success!
-                err = register_user(username, password)
-                if err:
-                    message = err
-                    status_code = 400
-                else:
-                    username = ""
-                    password = ""
-                    confirm_password = ""
-                    message = "Registration successful! You can login now."
-                    status_code = 200
-            return (
-                render_template(
-                    "old/auth.html",
-                    register_username=username,
-                    register_password=password,
-                    register_confirm_password=confirm_password,
-                    register_message=message,
-                    register_has_error=status_code != 200,
-                ),
-                status_code,
-            )
-
-        else:  # action == 'login'
-            username = request.form["username"].strip()
-            password = request.form["password"]
-            remember = request.form.get("remember") == "on"
-            if not (username and password):
-                message = "Empty field(s) spotted. Protip: spaces don't count."
-                status_code = 400
-            else:
-                user_id = verify_username_password(username, password)
-                if user_id is None:
-                    message = "Wrong username/password combination."
-                    status_code = 400
-                else:  # success!
-                    resp = redirect(request.args.get("next", url_for("home_view")))
-                    session.permanent = remember
-                    session["user"] = {"username": username, "id": user_id}
-                    return resp
-
-            return (
-                render_template(
-                    "old/auth.html",
-                    login_username=username,
-                    login_password=password,
-                    login_remember=remember,
-                    login_message=message,
-                    login_has_error=status_code != 200,
-                ),
-                status_code,
-            )
-
-    # Just a plain ol' GET request:
-    return render_template("old/auth.html")
-
-
-@app.route("/search")
-def search_view():
-    query = request.args.get("q", "").strip()
-    results = {}
-    if query:
-        results = search_title_all_sites(query)
-
-    if "mangadex" in results:
-        for title in results["mangadex"]:
-            title["thumbnail"] = url_for(
-                "proxy_view", b64_url=_encode_proxy_url(title["thumbnail"])
-            )
-    return render_template("old/search.html", results=results, query=query)
-
-
-@app.route("/proxy/<b64_url>")
-def proxy_view(b64_url):
-    """Fine I'll do it"""
-    url = _decode_proxy_url(b64_url)
-    if not _is_manga_img_url(url):
-        print("Invalid img url:", url)
-        return "Nope", 400
-    md_resp = requests.get(url)
-    resp = make_response(md_resp.content, md_resp.status_code)
-    resp.headers["Content-Type"] = md_resp.headers["Content-Type"]
-    return resp
-
-
 def _encode_proxy_url(url):
     return base64.urlsafe_b64encode(url.encode()).decode()
 
@@ -231,39 +80,38 @@ def _is_manga_img_url(
     return pattern.match(url)
 
 
-@app.route("/import", methods=["GET", "POST"])
-@require_login
-def import_view():
-    if request.method == "POST":
+@app.route("/proxy/<b64_url>")
+def proxy_view(b64_url):
+    """
+    Cached proxy for images (manga cover/page). Motivations:
+        - get around source site's hotlinking protection
+        - keep working even when source site is down
+        - be a polite netizen in general
+    """
+    url = _decode_proxy_url(b64_url)
+    if not _is_manga_img_url(url):
+        print("Invalid img url:", url)
+        return "Nope", 400
 
-        # check if the post request has the file part
-        if "tachiyomi" not in request.files:
-            flash("No file part")
-            return redirect(request.url)
-        file = request.files["tachiyomi"]
+    cached_file_path = Path(config.PROXY_CACHE_DIR) / b64_url
+    cached_headers_path = cached_file_path.with_suffix(".headers.json")
 
-        # if user does not select file, browser also
-        # submits an empty part without filename
-        if file.filename == "":
-            flash("No selected file")
-            return redirect(request.url)
+    if not (storage.exists(cached_file_path) and storage.exists(cached_headers_path)):
+        md_resp = requests.get(url)
+        status_code = md_resp.status_code
+        body = md_resp.content
+        headers = {"Content-Type": md_resp.headers["content-type"]}
+        if status_code == 200:
+            storage.save(cached_headers_path, json.dumps(headers).encode())
+            storage.save(cached_file_path, md_resp.content)
+    else:
+        status_code = 200
+        body = storage.read(cached_file_path)
+        headers = json.loads(storage.read(cached_headers_path))
 
-        if file:
-            text = file.read()
-            site_title_pairs = read_tachiyomi_follows(text)
-            if site_title_pairs is None:
-                flash("Malformed input file.")
-                return redirect(request.url)
+    headers["Cache-Control"] = "max-age=31536000"
 
-            # First fetch & save titles if they're not already in db
-            ensure_titles(site_title_pairs)
-
-            # Then follow them all
-            import_follows(session["user"]["id"], site_title_pairs)
-
-            flash(f"Added {len(site_title_pairs)} follows.")
-
-    return render_template("old/import.html")
+    return body, status_code, headers
 
 
 def read_tachiyomi_follows(text: str) -> List[Tuple[str, str]]:
@@ -303,11 +151,20 @@ def read_tachiyomi_follows(text: str) -> List[Tuple[str, str]]:
 
 
 def ensure_titles(site_title_pairs: List[Tuple[str, str]]):
-    new_titles = [
-        (site, title_id)
-        for site, title_id in site_title_pairs
-        if load_title(site, title_id) is None  # again, n+1 queries are fine in sqlite
-    ]
+    """
+    Fetch and save titles that are not already in db.
+    Returns a list of title dicts, both old and new.
+    """
+    title_dicts = []
+    new_titles = []
+
+    for site, title_id in site_title_pairs:
+        existing_title = load_title(site, title_id)
+        if existing_title is None:  # again, n+1 queries are fine in sqlite
+            new_titles.append((site, title_id))
+        else:
+            title_dicts.append(existing_title)
+
     print(f"Fetching {len(new_titles)} new titles out of {len(site_title_pairs)}.")
     with ThreadPoolExecutor(max_workers=3) as executor:
         futures = [
@@ -317,11 +174,9 @@ def ensure_titles(site_title_pairs: List[Tuple[str, str]]):
             title = future.result()
             save_title(title)
             print(f"Saved {title['site']}: {title['name']}")
+            title_dicts.append(title)
 
-
-"""
-New Mithril-based SPA views follow
-"""
+    return title_dicts
 
 
 @app.route("/")
@@ -330,6 +185,7 @@ New Mithril-based SPA views follow
 @app.route("/f")
 @app.route("/s")
 @app.route("/s/<query>")
+@app.route("/i")
 def home_view(query=None):
     return render_template("spa.html")
 
@@ -476,7 +332,11 @@ def api_login():
         return jsonify({"message": "Wrong username/password combination."}), 401
 
     token = create_token(user_id, remember)
-    return jsonify({"user_id": user_id, "token": token}), 200
+    resp = make_response(jsonify({"user_id": user_id, "token": token}))
+    resp.set_cookie(
+        "token", token, max_age=31536000 if remember else None, samesite="Strict"
+    )
+    return resp
 
 
 @app.route("/api/verify-token", methods=["GET"])
@@ -491,7 +351,9 @@ def api_logout():
     num_deleted = delete_token(request.token)
     if num_deleted != 1:
         return jsonify({"message": "Invalid token."}), 401
-    return "{}", 200
+    resp = make_response("{}", 200)
+    resp.set_cookie("token", "", expires=0)
+    return resp
 
 
 @app.route("/api/follows", methods=["GET"])
@@ -556,3 +418,36 @@ def api_read():
     # Also TODO: maybe a separate "read all from title" API would be cleaner & easier on
     # FE side.
     return {}
+
+
+@app.route("/api/import", methods=["POST"])
+@process_token(required=True)
+def api_import():
+    # check if the post request has the file part
+    if "tachiyomi" not in request.files:
+        return jsonify({"message": "No file provided"}), 400
+    file = request.files["tachiyomi"]
+
+    # if user does not select file, browser also
+    # submits an empty part without filename
+    if file.filename == "":
+        return jsonify({"message": "No selected file"}), 400
+
+    if file:
+        text = file.read()
+        site_title_pairs = read_tachiyomi_follows(text)
+        if site_title_pairs is None:
+            return jsonify({"message": "Malformed file."}), 400
+
+        # First fetch & save titles if they're not already in db
+        titles = ensure_titles(site_title_pairs)
+
+        # Then follow them all
+        import_follows(request.user_id, site_title_pairs)
+
+        # Mark all chapters as read too
+        for title in titles:
+            for chapter in title["chapters"]:
+                read(request.user_id, title["site"], title["id"], chapter["id"])
+
+        return jsonify({"message": f"Added {len(site_title_pairs)} follows."})
